import java.lang.Math;
import java.awt.geom.Point2D;
import java.util.*;
/**
 *The class <code>GlobalView</code> is responsible for all needed calculations with absolute and relative 
 *coordinates
 *
 *@author: Alla Korchminskaya
 *
 */
public class GlobalView
{
    //classatribute	
    /** parameter for the current side of the team's own goal */
    private char side = ' ';
    /** Vector which contains all the relative information about the seen Objects in the current cycle */
    private Vector visualInfoVector = new Vector();
    /**String which contains the current name of the team */
    private String teamName = "";
    /** parameter for the current view Direction in every cycle */
    double viewDir=0.0;


    /**
     *Creates a new <code>GlobalView</code> instance.
     *
     *@param side a side of the field, where  the players home zone is at 
     *
     *@param teamName a current name of the team
     */
    public GlobalView(char side, String teamName)
    {
	this.side=side;
	this.teamName=teamName;	  
    }


    /**
     *actualizes the Vector with the relative Information
     *
     *@param Vector which contains the relative Infomation about the seen Objects in the current cycle
     */
    public void actualize(Vector visualInfoVector)
    {
	this.visualInfoVector = visualInfoVector;
    }


    /**
     * calculates the absolute view direction of the player
     *
     *@param line a type of the line which is sighted by the player
     *
     *@param dir a realtive angle given from server
     */
    private  void absoluteViewDir(String line, double dir)
    {
	double temp=0.0;
	if (line.equals("l t")) 
	    {
		if (side=='l')
		    {
			if (dir<0)
			    {
				temp=180-dir;
				viewDir=temp-360;
			    }
			else
			    {
				viewDir=0-dir;
			    }
		    }
		
		else
		    {
		
			if(dir<0)
			    {
				viewDir=0-dir;
			    }
			else
			    {	
				viewDir=180-dir;
			    }
		    }
	    }
	if(line.equals("l b"))
	{
	    if (side=='l')
		{
		    if (dir<0)
			{
			    viewDir=0-dir;
			}
		    else
			{
			    viewDir=180-dir;
			}
		}
	    else
		{
		    if (dir<0)
			{
			    temp=180-dir;
			    viewDir=temp-360;
			}
		    else
			{
			    viewDir=0-dir;
			}
		}
	}
	if(line.equals("l l"))
	    {
		if(side=='l')
		    {
			if(dir<0)
			    {
				viewDir=180-dir-90;
			    }
			else
			    {
				viewDir=0-dir-90;
			    }
		    }
		else
		    {
			
			if(dir<0)
			    {
				temp=180-dir+90;
				viewDir=temp-360;
			    }
			else
			    {
				viewDir=0-dir+90;
			    }
		    }
	    }
	if(line.equals("l r"))
	    {
		if(side=='l')
		    {
			if(dir<0)
			    {
				temp=180-dir+90;
				viewDir=temp-360;
			    }
			else
			    {
				viewDir=0-dir+90;
			    }
		    }
		else
		    {
			
			if(dir<0)
			    {
				viewDir=180-dir-90;
			    }
			else
			    {
				viewDir=0-dir-90;
			    }	
		    }
	    }
	//System.out.println("Blickrichtung: " + viewDir);
    }


    /**
     * calculates the relative euklid coordinates
     *
     *@param direction a direction in which the object is seen
     *
     *@param distance a distancce in which the object is seen
     *
     *@return an <code>Point2D.Double</code> with realative coordinates
     */
    private Point2D.Double relativeCoordinates(double direction,double distance)
    { 
	//System.out.println("relative berechnung angefangen");

	Point2D.Double relCoordinates=new Point2D.Double(0,0);


	//System.out.println("Blickrichtung: " + viewDir);
	double absoluteAngle=viewDir+direction;

	double x=distance*(Math.cos(absoluteAngle*(Math.PI/180)));
	double y=distance*(Math.sin(absoluteAngle*(Math.PI/180)));
	relCoordinates.setLocation(x,y);
	return relCoordinates;
    }


    /**
     * calculates the absolute position of the seen object
     *
     *@param selfPosition <code>Point2D.Double</code> value which contains the calculated absolute position 
     *of the current player
     *
     *@param dir an direction in which the object is seen
     *
     *@param dist an distance in which the object is seen 
     *
     *@return an <code>Point2D.Double</code> value which contains the absolute positon of the object
     */
    private Point2D.Double absolutePosition(Point2D.Double selfPosition, double dir, double dist)
    {
        
	Point2D.Double absCoordinates=new Point2D.Double(0.0, 0.0);
	Point2D.Double relCoordinates = relativeCoordinates(dir, dist);
	double x=selfPosition.getX()+relCoordinates.getX();
	double y=selfPosition.getY()+relCoordinates.getY();
	absCoordinates.setLocation(x,y);
	return absCoordinates;
    }


    /**
     * calcutes the absolute position of the current player
     * 
     *@return an <code>Ponit2D.Double</code> value which contains the absolute position of the player
     */
    private Point2D.Double selfPosition()
    {
	Point2D.Double selfPosition = new Point2D.Double(0.0,0.0);

	HashMap h = new HashMap();

	Point2D.Double ftl50;
	Point2D.Double ftl40;
	Point2D.Double ftl30;
	Point2D.Double ftl20;
	Point2D.Double ftl10;
	Point2D.Double ft0;
	Point2D.Double ftr10;
	Point2D.Double ftr20;
	Point2D.Double ftr30;
	Point2D.Double ftr40;
	Point2D.Double ftr50;
	
	Point2D.Double frt30;
	Point2D.Double frt20;
	Point2D.Double frt10;
	Point2D.Double fr0;
	Point2D.Double frb10;
	Point2D.Double frb20;
	Point2D.Double frb30;
	
	Point2D.Double flt30;
	Point2D.Double flt20;
	Point2D.Double flt10;
	Point2D.Double fl0;
	Point2D.Double flb10;
	Point2D.Double flb20;
	Point2D.Double flb30;
	
	Point2D.Double fbl50;
	Point2D.Double fbl40;
	Point2D.Double fbl30;
	Point2D.Double fbl20;
	Point2D.Double fbl10;
	Point2D.Double fb0;
	Point2D.Double fbr10;
	Point2D.Double fbr20;
	Point2D.Double fbr30;
	Point2D.Double fbr40;
	Point2D.Double fbr50;
	
	Point2D.Double flt;
	Point2D.Double fct;
	Point2D.Double frt;
	Point2D.Double frb;
	Point2D.Double fcb;
	Point2D.Double flb;
	
	Point2D.Double fglt;
	Point2D.Double gl;
	Point2D.Double fglb;
	Point2D.Double fplt;
	Point2D.Double fplc;
	Point2D.Double fplb;
	Point2D.Double fgrt;
	Point2D.Double gr;
	Point2D.Double fgrb;
	Point2D.Double fprt;
	Point2D.Double fprc;
	Point2D.Double fprb;
	Point2D.Double fc;
	

	if (side == 'l')
	    {
		//absolute flag coordinates
	        ftl50 = new Point2D.Double(-50.0,-39.0);
		ftl40 = new Point2D.Double(-40.0,-39.0);
		ftl30 = new Point2D.Double(-30.0,-39.0);
		ftl20 = new Point2D.Double(-20.0,-39.0);
		ftl10 = new Point2D.Double(-10.0,-39.0);
		ft0 = new Point2D.Double(0.0,-39.0);
		ftr10 = new Point2D.Double(10.0,-39.0);
		ftr20 = new Point2D.Double(20.0,-39.0);
		ftr30 = new Point2D.Double(30.0,-39.0);
		ftr40 = new Point2D.Double(40.0,-39.0);
		ftr50 = new Point2D.Double(50.0,-39.0);
		
		frt30 = new Point2D.Double(57.5,-30.0);
		frt20 = new Point2D.Double(57.5,-20.0);
		frt10 = new Point2D.Double(57.5,-10.0);
		fr0 = new Point2D.Double(57.5,0.0);
		frb10 = new Point2D.Double(57.5,10.0);
		frb20 = new Point2D.Double(57.5,20.0);
		frb30 = new Point2D.Double(57.5,30.0);
		
		flt30 = new Point2D.Double(-57.5,-30.0);
		flt20 = new Point2D.Double(-57.5,-20.0);
		flt10 = new Point2D.Double(-57.5,-10.0);
		fl0 = new Point2D.Double(-57.5,0.0);
		flb10 = new Point2D.Double(-57.5,10.0);
		flb20 = new Point2D.Double(-57.5,20.0);
		flb30 = new Point2D.Double(-57.5,30.0);
		
		fbl50 = new Point2D.Double(-50.0,39.0);
		fbl40 = new Point2D.Double(-40.0,39.0);
		fbl30 = new Point2D.Double(-30.0,39.0);
		fbl20 = new Point2D.Double(-20.0,39.0);
		fbl10 = new Point2D.Double(-10.0,39.0);
		fb0 = new Point2D.Double(0.0,39.0);
		fbr10 = new Point2D.Double(10.0,39.0);
		fbr20 = new Point2D.Double(20.0,39.0);
		fbr30 = new Point2D.Double(30.0,39.0);
		fbr40 = new Point2D.Double(40.0,39.0);
		fbr50 = new Point2D.Double(50.0,39.0); 
		
		flt = new Point2D.Double(-52.5,-34.0);
		fct = new Point2D.Double(0.0,-34.0);
		frt = new Point2D.Double(52.5,-34.0);
		frb = new Point2D.Double(52.5,34.0);
		fcb = new Point2D.Double(0.0,34.0);
		flb = new Point2D.Double(-52.5,34.0);
		
		fglt = new Point2D.Double(-52.5,-7.01);
		gl = new Point2D.Double(-52.5,0.0);
		fglb = new Point2D.Double(-52.5,7.01);
		fplt = new Point2D.Double(-36.0,-20.175);
		fplc = new Point2D.Double(-36.0,0.0);
		fplb = new Point2D.Double(-36.0,20.175);
		fgrt = new Point2D.Double(52.5,-7.01);
		gr = new Point2D.Double(52.5,0.0);
		fgrb = new Point2D.Double(52.5,7.01);
		fprt = new Point2D.Double(36.0,-20.175);
		fprc = new Point2D.Double(36.0,0.0);
		fprb = new Point2D.Double(36.0,20.175);
		fc = new Point2D.Double(0.0,0.0);
		
		
	    }
	else
	    { 
		//absolute flag coordinates
	        ftl50 = new Point2D.Double(50.0,39.0);
		ftl40 = new Point2D.Double(40.0,39.0);
		ftl30 = new Point2D.Double(30.0,39.0);
		ftl20 = new Point2D.Double(20.0,39.0);
		ftl10 = new Point2D.Double(10.0,39.0);

		ft0 = new Point2D.Double(0.0,39.0);
		ftr10 = new Point2D.Double(-10.0,39.0);
		ftr20 = new Point2D.Double(-20.0,39.0);
		ftr30 = new Point2D.Double(-30.0,39.0);
		ftr40 = new Point2D.Double(-40.0,39.0);

		ftr50 = new Point2D.Double(-50.0,39.0);
		frt30 = new Point2D.Double(-57.5,30.0);
		frt20 = new Point2D.Double(-57.5,20.0);
		frt10 = new Point2D.Double(-57.5,10.0);
		fr0 = new Point2D.Double(-57.5,0.0);

		frb10 = new Point2D.Double(-57.5,-10.0);
		frb20 = new Point2D.Double(-57.5,-20.0);
		frb30 = new Point2D.Double(-57.5,-30.0);
      		flt30 = new Point2D.Double(57.5,30.0);
		flt20 = new Point2D.Double(57.5,20.0);

		flt10 = new Point2D.Double(57.5,10.0);
		fl0 = new Point2D.Double(57.5,0.0);
		flb10 = new Point2D.Double(57.5,-10.0);
		flb20 = new Point2D.Double(57.5,-20.0);
		flb30 = new Point2D.Double(57.5,-30.0);
	
		fbl50 = new Point2D.Double(50.0,-39.0);
		fbl40 = new Point2D.Double(40.0,-39.0);
		fbl30 = new Point2D.Double(30.0,-39.0);
		fbl20 = new Point2D.Double(20.0,-39.0);
		fbl10 = new Point2D.Double(10.0,-39.0);

		fb0 = new Point2D.Double(0.0,-39.0);
		fbr10 = new Point2D.Double(-10.0,-39.0);
		fbr20 = new Point2D.Double(-20.0,-39.0);
		fbr30 = new Point2D.Double(-30.0,-39.0);
		fbr40 = new Point2D.Double(-40.0,-39.0);

		fbr50 = new Point2D.Double(-50.0,-39.0); 
		flt = new Point2D.Double(52.5,34.0);
		fct = new Point2D.Double(0.0,34.0);
		frt = new Point2D.Double(-52.5,34.0);
		frb = new Point2D.Double(-52.5,-34.0);

		fcb = new Point2D.Double(0.0,-34.0);
		flb = new Point2D.Double(52.5,-34.0);
		fglt = new Point2D.Double(52.5,7.01);
		gl = new Point2D.Double(52.5,0.0);
		fglb = new Point2D.Double(52.5,-7.01);

		fplt = new Point2D.Double(36.0,20.175);
		fplc = new Point2D.Double(36.0,0.0);
		fplb = new Point2D.Double(36.0,-20.175);
		fgrt = new Point2D.Double(-52.5,7.01);
		gr = new Point2D.Double(-52.5,0.0);

		fgrb = new Point2D.Double(-52.5,-7.01);
		fprt = new Point2D.Double(-36.0,20.175);
		fprc = new Point2D.Double(-36.0,0.0);
		fprb = new Point2D.Double(-36.0,-20.175);
		fc = new Point2D.Double(0.0,0.0);
	    }
	//load the absolute coordinates of the flags in to the hashmap
	h.put("f t l 50", ftl50);
	h.put("f t l 40", ftl40);
	h.put("f t l 30", ftl30);
	h.put("f t l 20", ftl20);
	h.put("f t l 10", ftl10);

	h.put("f t 0", ft0);
	h.put("f t r 10", ftr10);
	h.put("f t r 20", ftr20);
	h.put("f t r 30", ftr30);
	h.put("f t r 40", ftr40);

	h.put("f t r 50", ftr50);
	h.put("f l t 30", flt30);
	h.put("f l t 20", flt20);
	h.put("f l t 10", flt10);
	h.put("f l 0", fl0);

	h.put("f l b 10", flb10);
	h.put("f l b 20", flb20);
	h.put("f l b 30", flb30);
	h.put("f b l 50", fbl50);
	h.put("f b l 40", fbl40);

	h.put("f b l 30", fbl30);
	h.put("f b l 20", fbl20);
	h.put("f b l 10", fbl10);
	h.put("f b 0", fb0);
	h.put("f b r 10", fbr10);

	h.put("f b r 20", fbr20);
	h.put("f b r 30", fbr30);
	h.put("f b r 40", fbr40);
	h.put("f b r 50", fbr50);
	h.put("f r t 30", frt30);

	h.put("f r t 20", frt20);
	h.put("f r t 10", frt10);
	h.put("f r 0", fr0);
	h.put("f r b 10", frb10);
	h.put("f r b 20", frb20);

	h.put("f r b 30", frb30); 
	h.put("f l t", flt);
	h.put("f c t", fct);
	h.put("f r t", frt);
	h.put("f g r t", fgrt);

	h.put("g r", gr);
	h.put("f g r b", fgrb);
	h.put("f r b", frb);
	h.put("f c b", fcb);
	h.put("f l b", flb);

	h.put("f g l b",fglb);
	h.put("g l", gl);
	h.put("f g l t", fglt);
	h.put("f p l t", fplt);
	h.put("f p l c", fplc);

	h.put("f p l b", fplb);
	h.put("f p r b", fprb);
	h.put("f p r c", fprc);
	h.put("f p r t", fprt);
	h.put("f c", fc);
    
	String lineNameMin=" ";
	String flagNameMin=" ";
       	double lineDistMin=-1000.0;
       	double flagDistMin=999.0;
	double lineDirMin=0.0;
	double flagDirMin=0.0;
	for (Enumeration visualInfoList=visualInfoVector.elements(); visualInfoList.hasMoreElements();)
	    {
		VisualInfo vi = (VisualInfo)visualInfoList.nextElement();
                if(vi.name().startsWith("l"))
		    {
	      		if(vi.distance()>lineDistMin)
			    {
				lineDistMin=vi.distance();
				lineNameMin=vi.name();
				lineDirMin=vi.direction();
			    }
		    }
		if(vi.name().startsWith("f"))
		    {
			if(vi.distance()<flagDistMin)
			    {
				flagDistMin=vi.distance();
				flagNameMin=vi.name();
				flagDirMin=vi.direction();
			    }
		    }
	    }
	//wenn keine flags gesehen
	absoluteViewDir(lineNameMin,lineDirMin);

	if((flagNameMin.equals(" ")) && (lineNameMin.equals(" ")))
	    { 
		double x = 999.0;
		double y = 999.0;
		selfPosition.setLocation(x,y);
		return selfPosition;
	    }
	else
	    {

	//System.out.println("flagNameMin: " + flagNameMin);
	//System.out.println("flagDistMin: " + flagDistMin);
	//System.out.println("flagDirMin: " + flagDirMin);
	//System.out.println("lineNameMin: " + lineNameMin);
	//System.out.println("lineDistMin: " + lineDistMin);
	//System.out.println("lineDirMin: " + lineDirMin);

	Point2D.Double relCoordinates = new Point2D.Double(0.0,0.0);
	relCoordinates =  relativeCoordinates(flagDirMin, flagDistMin);

	//System.out.println("relative X-Coordinate von flag: " + relCoordinates.getX());
	//System.out.println("relative Y-Coordinate von flag: " + relCoordinates.getY());

	Point2D.Double flagCoordinates  = new Point2D.Double(0.0,0.0); 
	
	flagCoordinates =  (Point2D.Double)h.get(flagNameMin);
	//System.out.println("absolute X-Coordinate von flag: " + flagCoordinates.getX());
	//System.out.println("absolute Y-Coordinate von flag: " + flagCoordinates.getY());

	double x=flagCoordinates.getX()-relCoordinates.getX();
	double y=flagCoordinates.getY()-relCoordinates.getY();
	selfPosition.setLocation(x,y);
	return selfPosition;
	    }
    }


    /**
     * goes 2 times through the the visualInfoVector and calculates the absolute position of the player, 
     *the absolute position of all other players that he see and of the ball if he sees it.
     *
     *@return an <code>Point2D.Double</code> value which contains the absolute position of the player
     */   
    public Point2D.Double initialize()
    {    
	//	double flagDistMin=999.0;
    
	Point2D.Double selfPosition=new Point2D.Double(999.0,999.0);
	selfPosition = selfPosition();
	//System.out.println("selfposition("+selfPosition.getX()+","+selfPosition.getY()+")");

	if(selfPosition.getX()==999.0)
	    {
		//System.out.println("KANN DIE EIGENE POSITION NICHT BERECHNEN!!!!");
	    }
	else
	    {
		for (Enumeration visualInfoList=visualInfoVector.elements(); visualInfoList.hasMoreElements();)
		    {
			VisualInfo vi = (VisualInfo)visualInfoList.nextElement();
			if(vi.name().startsWith("p"))
			    {
				Point2D.Double playerPosition = new Point2D.Double(0.0,0.0);
				playerPosition = absolutePosition(selfPosition, vi.direction(), vi.distance());
				vi.setX(playerPosition.getX());
				vi.setY(playerPosition.getY());
				//System.out.println("playerposition("+vi.getX()+","+vi.getY()+")");
			    }
			if(vi.name().startsWith("b"))
			    {
				Point2D.Double ballPosition = new Point2D.Double(0.0,0.0);
				ballPosition = absolutePosition(selfPosition, vi.direction(), vi.distance());
				//System.out.println("ballDirection "+vi.direction());
				//System.out.println("ballDistance "+vi.distance());
				//System.out.println("ballposition("+ballPosition.getX()+","+ballPosition.getY()+")");
				vi.setX(ballPosition.getX());
				vi.setY(ballPosition.getY());
			    }  
		    }
	    }
	return selfPosition;
    }


    /**
     *tests, if there are players within the kickable margine of the ball(currently not used)
     *
     *@param an <code>VisualInfo</code> value which contains all the informatio of the ball
     *
     *@return int
     */
    public int getBallProperty(VisualInfo ball)
    {
	final double kickableMargin = 0.7;
	int returnValue=0;
	double dist=0;
	for (Enumeration visualInfoList=visualInfoVector.elements(); visualInfoList.hasMoreElements();)
	    {
		VisualInfo vi = (VisualInfo)visualInfoList.nextElement();
		
		if(vi.name().startsWith("p"))
		    {
			dist=Math.sqrt(Math.pow(ball.getX()-vi.getX(),2.0) + Math.pow(ball.getY()-vi.getY(),2));
			if(dist<kickableMargin)
			    {
				
				if(vi.name().startsWith("p "+teamName))
				    {
					
					if(returnValue==2)
					    {
						returnValue=0;
						break;
					    }
					else
					    {returnValue=1;}
				    }
				else
				    {
					if(returnValue==1)
					    {
						returnValue=0;
						break;
					    }
					else
					    {returnValue=2;}
				    }
			    }
		    }
	    }
	return returnValue;
    }


    /**
     * calcutes the turn angle in order to turn to a certain absolute position
     *
     *@param selfPosition an <code>Point2D.Double</code> which contains the absolute position of the player
     *
     *@param x the x-coordinates of the turn position
     *
     *@param y the y-coordinates of the turn position
     *
     *@param headAngle the head angle of the player
     *
     *@return an<code>double</code> value which contains the turn angle
     */        
    public double getTurnDir(Point2D.Double selfPosition, double x, double y, double headAngle)
    {
	double turnAngle=0.0;
	double radiantAngle = Math.atan((Math.abs(selfPosition.getX()-x))/(Math.abs(selfPosition.getY()-y)));

	double angle = radiantAngle * (180/Math.PI);

	//System.out.println("angle: " + angle);
	if(side=='l')
	    {
		if((selfPosition.getX()>x) && (selfPosition.getY()>y))
		    {
			turnAngle = -viewDir-90-angle;
		    }
		if((selfPosition.getX()<x) && (selfPosition.getY()>y))
		    {
			turnAngle = -viewDir-90+angle;
		    }
		if((selfPosition.getX()>x) && (selfPosition.getY()<y))
		    {
			turnAngle = -viewDir+90+angle;
		    }
		if((selfPosition.getX()<x) && (selfPosition.getY()<y))
		    {
			turnAngle = -viewDir+90-angle;
		    }
		if((selfPosition.getX()==x) && (selfPosition.getY()>y))
		    {
			turnAngle = -viewDir-90;
		    }
		if((selfPosition.getX()==x) && (selfPosition.getY()<y))
		    {
			turnAngle = -viewDir+90;
		    }
		if((selfPosition.getX()<x) && (selfPosition.getY()==y))
		    {
			turnAngle = viewDir;
		    }
		if((selfPosition.getX()>x) && (selfPosition.getY()==y))
		    {
			turnAngle = -viewDir+180;
		    }
	    }
	else
	    {
		if((selfPosition.getX()>x) && (selfPosition.getY()>y))
		    {
			turnAngle = -viewDir-90-angle;
		    }
		if((selfPosition.getX()<x) && (selfPosition.getY()>y))
		    {
			turnAngle = -viewDir-90+angle;
		    }
		if((selfPosition.getX()>x) && (selfPosition.getY()<y))
		    {
			turnAngle = -viewDir+90+angle;

		    }
		if((selfPosition.getX()<x) && (selfPosition.getY()<y))
		    {
			turnAngle = -viewDir+90-angle;
		    }
		if((selfPosition.getX()==x) && (selfPosition.getY()>y))
		    {
			turnAngle = -viewDir-90;
		    }
		if((selfPosition.getX()==x) && (selfPosition.getY()<y))
		    {
			turnAngle = -viewDir+90;
		    }
		if((selfPosition.getX()<x) && (selfPosition.getY()==y))
		    {
			turnAngle = -viewDir;
		    }
		if((selfPosition.getX()>x) && (selfPosition.getY()==y))
		    {
			turnAngle = -viewDir-180;
		    }
	    }
		double temp = headAngle + turnAngle;
		//System.out.println("temp" + temp);
		double returnAngle =0;
		if(temp<-180)
		    {
			returnAngle = temp +360;
		    }
		if(temp>180)
		    {
			returnAngle = temp-360;
		    }
		if((temp>=-180) && (temp<=180))
		    {
			returnAngle = temp;
		    }

	return returnAngle;
     }
    /*  
    public static void main( String[] args )
    {
	Vector visualInfoVector = new Vector();
       
	VisualInfo flag1 = new VisualInfo(0,"f c",44.7,-43.0,0.0,0.0);
	VisualInfo flag2 = new VisualInfo(0,"f c b", 20.9,3.0,0.0,0.0);
      	VisualInfo flag3 = new VisualInfo(0,"f r t",103.5, -26.0,0.0,0.0);
       	//VisualInfo flag4 = new VisualInfo(0,"f t r 10", 27.7, -44.0,0.0,0.0);
	//VisualInfo flag5 = new VisualInfo(0,"f p l c" ,27.9, -41.0,0.0,0.0);





	VisualInfo line1 = new VisualInfo(0,"l r", 77.5, -70.0,0.0,0.0);
 VisualInfo line2 = new VisualInfo(0,"l b", 17.5, 20.0,0.0,0.0);
	VisualInfo ball = new VisualInfo(0,"b", 44.7,-43.0,0.0,0.0);
	VisualInfo player1 = new VisualInfo(0,"p team",47.7,-7.0,0.0,0.0);
	//VisualInfo player2 = new VisualInfo(0,"p team",15.0, 15.5,0.0,0.0);
	

       	visualInfoVector.add(flag1);
	visualInfoVector.add(flag2);
	visualInfoVector.add(flag3);
       	//visualInfoVector.add(flag4);
	//visualInfoVector.add(flag5);
	
	visualInfoVector.add(line1);
	visualInfoVector.add(line2);
	visualInfoVector.add(ball);
	visualInfoVector.add(player1);
	//visualInfoVector.add(player2);
      
	GlobalView globalView = new GlobalView('r',"winer");
	globalView.actualize(visualInfoVector);
       	Point2D.Double selfPosition = new Point2D.Double(0.0,0.0);
	//selfPosition = globalView.selfPosition();
	//System.out.println("selfposition("+selfPosition.getX()+","+selfPosition.getY()+")");
	//Point2D.Double theball = new Point2D.Double(0.0,0.0);
	//theball = globalView.absolutePosition(selfPosition,-36.0,27.1);
	//System.out.println("ballPosition("+theball.getX()+","+theball.getY()+")");
	//double turn = globalView.getTurnDir(selfPosition, 52.5, 0.0, 0.0);
	//System.out.println("turnAngle: " + turn);
	
	selfPosition = globalView.initialize();
	double turn = globalView.getTurnDir(selfPosition, 52.5, 0.0, 0.0);
	 //System.out.println("turnAngle: " + turn);
	for (Enumeration visualInfoList=visualInfoVector.elements(); visualInfoList.hasMoreElements();)
	    {
		VisualInfo vi = (VisualInfo)visualInfoList.nextElement();
		
		if(vi.name().startsWith("b"))
		    {
			//System.out.println("ball x-Cordinate" + vi.getX());
			//System.out.println("ball x-Cordinate" + vi.getY());

			int test = globalView.getBallProperty(vi);
			//System.out.println("test: " + test);
		    }
	    }
	

    }
*/

}
	


 







































