/*The class <code>Parser</code> enables Player to use methods in order to 
 *inform the communication class about the commands
 *
 *@author: Sanaz Jalayer
 */
public class Parser
{
    /**a new <code>Event</code> instance*/
    private Event event;

    /** current side of the team's own goal */
    private char side = ' ';

    /** a new <code>Communication</code> instance */
    public Communication communication;


    /** 
     *creates a new <code>Parser</code> instance
     *
     *@param Communication a <code>Communication</code> instance 
     */
    public Parser(Communication communication)
    {
	this.communication = communication;

    }


    /**
     *sends an init-command to the server
     *
     *@param teamName the current name of the team
     *@param playerNumber a number of the current player
     */
    public void init(String teamName, int playerNumber)
    {
	if(playerNumber == 11)
	    communication.send(new String("(init " + teamName + " (version 7.0)"+ " (goalie))"));


	else communication.send(new String("(init " + teamName + " (version 7.0))"));
	// communication.send(new String("(init "+ teamName + "version 7.0))"));
    }


    /**
     *receives the incoming message from server and indicates which type of the event it is 
     *
     *@return an <code>Event</code> object containing server information 
     */ 
    public Event event()
    {
	String message = "null";
	event = null;

	while (event== null)
	    {
		message = communication.receive();
	 


	if(message.startsWith("(init")) //finde heraus, welche seite
	    {
		//System.out.println("INIT ANALYSE in PARSER !!!!!!!!!!!!!!!!!!!!!!");
		if(message.substring(6).startsWith("l"))
		    {
			side = 'l';
		    }
		
		else if (message.substring(6).startsWith("r"))
		    {
			side = 'r';
		    }
	    }

	if (message.startsWith("(see"))
	    {
		event = new VisualEvent(message);
	    }
	else if(message.startsWith("(sense_body"))
	    { 
		event = new BodyEvent(message);
	    }

	else if(message.startsWith( "(hear"))
	    {
		event = new AcousticEvent(message);
	    }
	    }
	return event;
	    
    }



    /**
     * gives a runnig impulse with a given power
     *
     *@param power a <code>double</code> value which contains the power of the impulse
     */
   public void dash(double power)
    {
        communication.send(new String("(dash " + power + ")"));
    }


    /**
     * Kicks into a given directions with a given power
     *
     *@param power a <code>double</code> value spesifying the power of the kick
     *
     *@param direction a <code>double</code> value giving the direction of the kick
     */
  public void kick(double power, double direction)
    {
        communication.send(new String("(kick " + power + " " + direction + ")"));
    }


    /**
     *Places the player to a give position on the field
     * 
     *@param x a <code>double</code> value which gives the position in x direction
     *
     *@param y a <code>double</code> value which gives the position in y direction
     */
   public void move(double x, double y)
    {
        communication.send(new String("(move " + x + " " + y + ")"));
    }


    /**
     *Turns the body of the player on the field
     *
     *@param moment a <code>double</code>value which gives the angle (in degree) relative 
     *to the player's body in which he wants to turn  
     */
   public void turn(double moment)
    {
        
	communication.send(new String("(turn " + moment + ")"));
	
    }


    /**
     *Turns the head of the player on the field
     *
     *@param moment a <code>double</code> value which gives the angle (in degree) relative 
     *to the player's body in which he wants to turn
     */
    public void turnNeck(double moment)
    {
	communication.send(new String("(turn_neck " + moment + ")"));

    }


    /**
     *Catches the ball in the given direction
     *
     *@param direction a<code>double</code> value which gives the angle (in degree)  relative
     * to the goalie's body in which he wants to catch the ball
     */
   public void catchBall(double direction)
       {
	   communication.send(new String("(catch " +direction+ ")"));
       }


    /**
     *Says a message, which other players on the field can hear 
     *
     *@param message a <code>String</code> value which gives the player' s message that he wants to say
     */
   public void say(double message)
       {
	   communication.send(new String("(say " +message+ ")"));
       }

    /**
     *The current side of the team's own goal
     *
     *@return a <code>side</code> value which contains the current side of the teams's own goal
     */ 
   public char side()
    {
	return side;
    }
    // debug public static void main (String []args)
    //{System.out.println("hallo!!");}



}






















