import java.util.*;
import java.awt.geom.Point2D;

/**This is the central class.  Here, all visual information
 *is evaluated and converted into decisions and actions 
 *according to the strategy
 *
 * @author Alexander Bast
 *
 */
public class Player extends Thread
{

    /** the  module of the client */
    private Parser  parser;

    /** the name of the team (for initialization) */
    private String teamName;

    /** the number of the player */
    private int uniformNumber;

    /** the number current Thread */
    private int actualThreadNumber;

    /** used play mode before kick off */
    final private int before_kick_off = 0;

    /** used play mode play on */
    final private int play_on = 1;

    /** used play mode free kick */
    final private int free_kick = 2;

    /** used play mode after goal */
    final private int after_goal = 3;

    /** used play mode own goal kick */
    final private int own_goal_kick = 4;

    /** counter used for moving players */
    private int zaehler = 0;

    /** counter used for moving players */
    private boolean geturnt = false;

    /** represtens the current game status */
    private int status = before_kick_off;

    /** the current information regarding the ball*/
    private VisualInfo ballInfo = new VisualInfo(-1,"",0.0,0.0,0.0,0.0);
    
    /** the current information regarding the own goal*/
    private VisualInfo ownGoalInfo = new VisualInfo(-1,"",0.0,0.0,0.0,0.0);;

    /** the current information regarding the opponent goal*/
    private VisualInfo opponentGoalInfo = new VisualInfo(-1,"",0.0,0.0,0.0,0.0);

    /** closest opponent player in the current cycle*/
    private VisualInfo playerInfo1 = new VisualInfo(-1,"",0.0,0.0,0.0,0.0);

    /** 2nd closest opponent player in the current cycle*/
    private VisualInfo playerInfo2 = new VisualInfo(-1,"",0.0,0.0,0.0,0.0);

    /** 3nd closest opponent player in the current cycle*/
    private VisualInfo playerInfo3 = new VisualInfo(-1,"",0.0,0.0,0.0,0.0);

    /**a new <code>AcousticInfo</code> instance*/
    private AcousticInfo acousticInfo;
    
    /**a new <code>BodyInfo</code> instance*/
    private BodyInfo bodyInfo;

    /**a new <code>Communication</code> instance*/
    private Communication communication;

    /** contains the side our team plays on*/
    private char side = ' ';

    /** contains the side the opponent team plays on*/
    private char opponentSide;

     /** contains a quotation as char*/
    private char anf = '"';

    /** helps to find out opponent players*/
    private String anf2;

    /**a new <code>GlobalView</code> instance*/
    private GlobalView globalView;
 
    /**a new <code>Event</code> instance*/
    private Event event;

    /**a new <code>SuperPlayer</code> instance*/
    private SuperPlayer superPlayer;

    /** the actual cycle of last visual event received */
    private int actualTime = -1;

    /** counter to create globalView only once */
    private int globalViewCounter = 0;

    /**initalizes the visualisation*/
    private int visu = 0;

    /**contains the absolute position of the player*/
    private Point2D.Double selfPosition = new Point2D.Double(0.0,0.0);

    /**a new <code>Graph</code> instance*/
    private Graph graph;

    /**
     * Creates a new <code>Player</code> instance.
     *
     * @param servername a <code>String</code> value specifying the name of the server
     *
     * @param serverport an <code>int</code> value specifying the port
     *
     * @param teamName a <code>String</code> value is the name of the team
     */
    public Player(String servername, int serverport, String teamName)
    {
	this.teamName = teamName;

	communication = new Communication(servername, serverport);
	parser = new Parser(communication);
	//System.out.println("Player Konstruktor");
    }

    /**
     * Creates a new <code>Player</code> instance.
     *
     * @param servername a <code>String</code> value specifying the name of the server
     *
     * @param serverport an <code>int</code> value specifying the port
     *
     * @param teamName a <code>String</code> value is the name of the team
     *
     * @param visu an <code>int</code> value specifying the info to start the visualisation
     */
    public Player(String servername, int serverport, String teamName, int visu)
    {
	this.teamName = teamName;
	this.visu = visu;

	communication = new Communication(servername, serverport);
	parser = new Parser(communication);
    }


    /**
     * This method overwrites <code>run()</code> from <code>Thread</code>. It is
     * executed when <code>start()</code> is called for the <code>Player</code>
     * Thread
     */
    public void run()
    {

	actualThreadNumber = Integer.parseInt(Thread.currentThread().getName().substring(1));

	anf2 = new String(String.valueOf(anf));

	try
	    {
		Thread.sleep(2500*actualThreadNumber);
	    }
	catch(InterruptedException q)
	    {
		
	    }
	if (visu == 1)
	    {
		graph = new Graph(actualThreadNumber);
	    }
	
	//System.out.println("Run gestartet "+actualThreadNumber);

	// register player
	parser.init(teamName,actualThreadNumber);

	Event event = null;

	// and play
	while (true)
	    {
		// receive event - block until one arrives
		event = parser.event();


		while(side == ' ')
		    {
			side = parser.side();
			//System.out.println("GEFANGEN IN WHILE-SCHLEIFE!!!!!!!!!!!");
		    }

		if (side == 'l')
		    {
			opponentSide = 'r';
		    }
		else
		    {
			opponentSide = 'l';
		    }

		//System.out.println("Seite="+side+"!");



		// update information about the world
		if (event.isVisualEvent())
		    {
			ballInfo.setName("");
			playerInfo1.setName("");
			playerInfo2.setName("");
			playerInfo3.setName("");

			//FUER ALLA!!!!!
			if(globalViewCounter == 0)
			    {
				globalView = new GlobalView(side,teamName);
				globalViewCounter++;
			    }
			else
			    {
				globalView.actualize(((VisualEvent)event).getVector());
			    }

			selfPosition = globalView.initialize();

			if (visu == 1)
			  {
			    graph.actualize(((VisualEvent)event).getVector(),teamName,side,selfPosition);
              }

			for (Enumeration visualInfoVector =((VisualEvent)event).elements(); visualInfoVector.hasMoreElements();)
			    {
				VisualInfo visualVectorSearch = (VisualInfo)visualInfoVector.nextElement();
				actualTime = visualVectorSearch.time();
				//System.out.println("VISUAL VECTOR " + visualVectorSearch.name());
				//check for the ball
				if (visualVectorSearch.name().startsWith("b"))
				    {
					if (ballInfo.time() < actualTime)
					    {
						ballInfo = visualVectorSearch;
					    }
				    }





				//check for oponents' goal
				if (visualVectorSearch.name().startsWith("g "+opponentSide))
				    {
					if (opponentGoalInfo.time() < actualTime)
					    {
						opponentGoalInfo = visualVectorSearch;
					    }
					else
					    {
						opponentGoalInfo.setName("");
					    }
				    }
				else
				    {
					if (visualVectorSearch.name().startsWith("g "+side))
					    {
						if (ownGoalInfo.time() < actualTime)
						    {
							ownGoalInfo = visualVectorSearch;
						    }
						else
						    {
							ownGoalInfo.setName("");
						    }
					    }

					else
					    {
						if (visualVectorSearch.name().startsWith("p"))
						    {
							if (!(visualVectorSearch.name().startsWith("p "+anf2+teamName)))
							    {
								//check for Players of own Team

								if ((playerInfo1.name().equals("")) || ((visualVectorSearch.distance()) < (playerInfo1.distance())))
								    {
									playerInfo3 = playerInfo2;
									playerInfo2 = playerInfo1;
									playerInfo1 = visualVectorSearch;
								    }
								else
								    if ((playerInfo2.name().equals("")) || ((visualVectorSearch.distance()) < (playerInfo2.distance())))
									{
									    playerInfo3 = playerInfo2;
									    playerInfo2 = visualVectorSearch;
									}
								    else
									{
									    if ((playerInfo3.name().equals("")) || ((visualVectorSearch.distance()) < (playerInfo3.distance())))
										{
										    playerInfo3 = visualVectorSearch;
										}
									}
							    }
							    
						    }
					    }
				    }

			    }
		    }
		else
		    {
			if (event.isAcousticEvent())
			    {

				for (Enumeration acousticInfoVector =((AcousticEvent)event).elements(); acousticInfoVector.hasMoreElements();)
				    {
					AcousticInfo acousticVectorSearch = (AcousticInfo)acousticInfoVector.nextElement();

					if (acousticVectorSearch.sender().equals("referee"))
					    {
						if (acousticVectorSearch.message().equals("before_kick_off"))
						    {
							status = before_kick_off;
						    }
						else
						    {
							if(acousticVectorSearch.message().equals("play_on"))
							    {
								status = play_on;
							    }
							else
							    {
								if((acousticVectorSearch.message().startsWith("offside_" + side)) || (acousticVectorSearch.message().startsWith("free_kick_" + side)))
								    {
									status = free_kick;
								    }
								else
								    {
									if(acousticVectorSearch.message().equals("goal_kick_"+side))
									    {
										status = own_goal_kick;
									    }
									else
									    {
										if(acousticVectorSearch.message().startsWith("goal_"))
										    {
											status = after_goal;
										    }
										else
										    {
											//guarantees Playing in all other Referee-Message-Cases
											status = play_on;
										    }
									    }
								    }
							    }
						    }
					    }
				    }
			    }
			else
			    {
				if (event.isBodyEvent())
				    {
					//System.out.println("................"+playerInfo1.name());
					
					for (Enumeration bodyInfoVector =((BodyEvent)event).elements(); bodyInfoVector.hasMoreElements();)
					    {
						BodyInfo bodyVectorSearch = (BodyInfo)bodyInfoVector.nextElement();
						
						bodyInfo = bodyVectorSearch;
					    }

					switch (status)
					    {
			    
					    case before_kick_off:

						superPlayer = new SuperPlayer(parser,side,globalView);

						superPlayer.setPositionStart(actualThreadNumber-1);

						//status = play_on;
						break;

					    case after_goal:

						superPlayer.setPositionStart(actualThreadNumber-1);

						//status = play_on;
						break;

					    case own_goal_kick:

						//  if (Thread.currentThread().getName().startsWith("G"))
//  						    {
//  							playGoalie();
//  						    }
						status = free_kick;

						//status = play_on;
						break;



					    case free_kick:

						if(ballInfo.getX() < -20.0)
						    {
							if (Thread.currentThread().getName().startsWith("G"))
							    {
								playGoalie();
							    }
						    }
						else
						    {
							if (Thread.currentThread().getName().startsWith("A"))
							    {
								playAttack();
							    }
							if (Thread.currentThread().getName().startsWith("M"))
							    {
								playMidfield();
							    }
							if (Thread.currentThread().getName().startsWith("D"))
							    {
								playDefense();
							    }
							//  if (Thread.currentThread().getName().startsWith("G"))
//  							    {
//  								playGoalie();
//  							    }
							
							//status = play_on;
						    }
						break;
						

					    case play_on:

						if (Thread.currentThread().getName().startsWith("A"))
						    {playAttack();}
						if (Thread.currentThread().getName().startsWith("M"))
						    {playMidfield();}
						if (Thread.currentThread().getName().startsWith("D"))
						    {playDefense();}
						if (Thread.currentThread().getName().startsWith("G"))
						    {playGoalie();}

						break;
					    }
				    }
			    }

		    }
	    }
    }

    /**
     * This method contains the strategy of the three attack players of our team
     */
    private void playAttack()
    {

	boolean ballSeen = false;
	
	if(ballInfo.name().equals(""))
	    {}
	else ballSeen = true;
	
	
	
	if(ballSeen)
	    {
		
		
		if(ballInfo.distance()+ballInfo.distChange() <= 0.7)   //kickmargin
		    {
			if(selfPosition.getX() != 999.0)
			    {
				
				
 				if( superPlayer.checkGoodGoalPosition(selfPosition) && (Math.abs(selfPosition.getY())) <= 12.0 )

				
				    {
					parser.kick(100,globalView.getTurnDir(selfPosition,52.5, 0.0, bodyInfo.getHeadAngle()));
				    }
				else
				    { 
					
				      parser.kick(100,globalView.getTurnDir(selfPosition,52.5, 0.0, bodyInfo.getHeadAngle()));
				    }
				    
			    }
			else
			    {
				parser.kick(100, opponentGoalInfo.direction());
				//System.out.println("OPPONENT GOAL INFO!!!");
			    }
		    }
	    }
	
	    
	    if(superPlayer.checkIfInDefArea(selfPosition, actualThreadNumber-1) == false)
		{
		    Point2D.Double defAr = superPlayer.getDefAr(actualThreadNumber-1);

		      if((zaehler == 0) )
  			{

		      parser.turn(globalView.getTurnDir(selfPosition,defAr.getX()+ 30.0,defAr.getY()-15.0, bodyInfo.getHeadAngle()));
			     zaehler++;


			     }

		      else
			  {
			      parser.dash(100);
					  //*superPlayer.getDistance(selfPosition, defAr.getX(),defAr.getY()));
			    zaehler = 0;
		      }


		}
	    else  //IN AREA
		{

		    if( !ballSeen || (Math.abs(ballInfo.direction()+ ballInfo.dirChange()) > 70.0))

		    {
			superPlayer.searchBall(ballInfo.direction(),ballInfo.dirChange(),bodyInfo.getHeadAngle(), ballInfo.distChange());


		    }

		    else  // ball gesehen
		    {



			if(superPlayer.ballInDefA(ballInfo, actualThreadNumber-1)== false)
			    {
				//System.out.println(" BALL NICHT IN MEINER HAELFTE" + actualThreadNumber);

				//  ball nahe tor renn hin
				if(ballInfo.getX() > 32.5)  
				    {
					if(zaehler == 0)
					    {

						parser.turn(ballInfo.direction()+ ballInfo.dirChange());
						zaehler++;
						    }
					else
					    {
					
						parser.dash(80*(ballInfo.distance()+ ballInfo.distChange()));
						zaehler = 0;
					    }

				    }

				else{ //ball  nicht vor tor
		
				    if(selfPosition.getX() > ballInfo.getX())
					{
					    if(superPlayer.checkIfInDefPoint(selfPosition, actualThreadNumber-1) == false)
						{
						    Point2D.Double coordinates = superPlayer.getStartPos(actualThreadNumber-1);
						    superPlayer. goToInitialPosition(globalView.getTurnDir(selfPosition,coordinates.getX(), coordinates.getY(),bodyInfo.getHeadAngle()), coordinates.getX(), coordinates.getY(), selfPosition);
						}
				    
					}
				//      if(superPlayer.checkIfInDefArea(selfPosition, actualThreadNumber-1) == false)
//  					{
//  					    Point2D.Double defAr = superPlayer.getDefAr(actualThreadNumber-1);
					    
//  					    if((zaehler == 0) )
//  						{
						
//  						    parser.turn(globalView.getTurnDir(selfPosition,defAr.getX()+ 30.0,defAr.getY()-15.0, bodyInfo.getHeadAngle()));
//  						    zaehler++;	
//  						}
					    
//  					    else
//  						{
//  						    parser.dash(100*superPlayer.getDistance(selfPosition, defAr.getX(),defAr.getY()));
//  						    zaehler = 0;
//  						}
//  					}
					    
				}
				
			    }			
			else //ball in AREA
			    {

				if(ballInfo.distance()+ ballInfo.distance() <=0.7)
				    {
					
					if(selfPosition.getX() != 999.0)
					    {
						if((superPlayer.checkGoodGoalPosition(selfPosition)) && (Math.abs(selfPosition.getY()) <= 12.0))
						    {
							parser.kick(100, globalView.getTurnDir(selfPosition, 52.5, 0.0,bodyInfo.getHeadAngle()));
						    }
						else 
						    {
							parser.kick(100, globalView.getTurnDir(selfPosition, 52.5, 0.0,bodyInfo.getHeadAngle()));
						    }
					    }
					else
					    {
						parser.kick(100, opponentGoalInfo.direction());
						//System.out.println("OPPONENT GOAL INFO!!!");
					    }
					
				    }				
				else     // balldistanz > 0.7
				    {
					if (ballInfo.distance() + ballInfo.distChange() < 5.0)
					    {

							if(Math.abs(bodyInfo.getHeadAngle()) < 0.5)
							    {
								parser.turnNeck(ballInfo.direction()+ballInfo.dirChange());
								parser.dash(100.0 * (ballInfo.distance()+ballInfo.distChange()));
							    }
						    
							else
							    {
								parser.turnNeck(-bodyInfo.getHeadAngle());
								parser.turn((bodyInfo.getHeadAngle())+(ballInfo.direction()+ ballInfo.dirChange()));
							    } 
    
						    
						

					    }
					else  //ballInfo
					    {
						 

							if (Math.abs(bodyInfo.getHeadAngle()) < 2.0)
							    {
								
								parser.turnNeck(ballInfo.direction()
 + ballInfo.dirChange());
								
								parser.dash(100.0* (ballInfo.distance()+ballInfo.distChange()));
							    }
						    
					    

					   
							else
							    {
								if(bodyInfo.getHeadAngle() < 0)
								    {
									parser.turnNeck(-bodyInfo.getHeadAngle());
									parser.turn((ballInfo.direction()+ ballInfo.dirChange() + bodyInfo.getHeadAngle()));
								    }
							
								else 
								    {
									parser.turnNeck(-(bodyInfo.getHeadAngle()));
									parser.turn(ballInfo.direction()+ballInfo.dirChange()+ bodyInfo.getHeadAngle());
								    }
							    
							
							//superPlayer.searchBall(ballInfo.direction(), bodyInfo.getHeadAngle());
						    }

						
						
					    }
					
				    }
			    }
		    }
		}
    }
    
	    
    /**
     * This method contains the strategy of the three midfield players of our team
     */
    private void playMidfield()
    {
	if (superPlayer.ballInOwnHalf(ballInfo))
	    {
		playDefense();
	    }
	else
	    {
		playAttack();
	    }
    }

  
    /**
     * This method contains the strategy of the four defense players of our team
     */



    private void playDefense()
    {

	if(ballInfo.name().equals("") || (Math.abs(ballInfo.direction() + ballInfo.dirChange()) > 70.0) )

		{
		    superPlayer.searchBall(ballInfo.direction(),ballInfo.dirChange() ,bodyInfo.getHeadAngle(), ballInfo.distChange());
		}



	    else //BALL GESEHEN
		{

				if(superPlayer.ballInDefA(ballInfo, actualThreadNumber-1)== false)
				    {
					//System.out.println(" BALL NICHT IN MEINER HAELFTE" + actualThreadNumber);


					if(superPlayer.checkIfInDefPoint(selfPosition, actualThreadNumber-1) == false)
					    {
						Point2D.Double coordinates = superPlayer.getStartPos(actualThreadNumber-1);
						superPlayer. goToInitialPosition(globalView.getTurnDir(selfPosition,coordinates.getX(), coordinates.getY(),bodyInfo.getHeadAngle()), coordinates.getX(), coordinates.getY(), selfPosition);
					    }
					//if(superPlayer.checkIfInDefArea(selfPosition, actualThreadNumber-1) == false)
					//  //  {





//  						//Point2D.Double defAr = superPlayer.getDefAr(actualThreadNumber-1);
						
//  						if((zaehler == 0) )
//  						    {
							
//  							parser.turn(globalView.getTurnDir(selfPosition,defAr.getX()+ 30.0,defAr.getY()-15.0, bodyInfo.getHeadAngle()));
//  							zaehler++;	
//  						    }
						
//  						else
//  						    {
//  							parser.dash(100*superPlayer.getDistance(selfPosition, defAr.getX(),defAr.getY()));
//  							zaehler = 0;
//  						    }
						
						
//  					    }

				
				

				    }


				else { //FALL BALL IN MEINEM BEREICH


					    if(ballInfo.distance()+ballInfo.distChange() <=0.7)
						{   
						    if(selfPosition.getX() != 999.0)
							{
							    parser.turnNeck(ballInfo.direction() + ballInfo.dirChange());
							    parser.kick(100, globalView.getTurnDir(selfPosition, 52.5, 0.0, bodyInfo.getHeadAngle()));
							   //   if((selfPosition.getY() < -10.0) && (selfPosition.getX() < 0.0))
//  								{
								   
//  								    parser.kick(100, globalView.getTurnDir(selfPosition, 10.0, -25.0, bodyInfo.getHeadAngle()));
								
//  								    //System.out.println(" GOALKICK MIT WINKEL " +   globalView.getTurnDir(selfPosition, 10.0, 25.0, bodyInfo.getHeadAngle()));
//  								}
//  							    else
//  								{
//  								    if((selfPosition.getY() > 10.0) && (selfPosition.getX() < 0.0))
//  									{
//  									    parser.kick(100, globalView.getTurnDir(selfPosition, 10.0,25.0, bodyInfo.getHeadAngle()));

//  									    //System.out.println(" GOALKICK MIT WINKEL " +   globalView.getTurnDir(selfPosition, 10.0, 25.0, bodyInfo.getHeadAngle()));

//  									}
								
							
								//      else
//  									{
//  									    parser.kick(100, globalView.getTurnDir(selfPosition, 52.5, selfPosition.getY(), bodyInfo.getHeadAngle()));
//  									    //System.out.println(" GOALKICK MIT WINKEL " +   globalView.getTurnDir(selfPosition, 10.0, 25.0, bodyInfo.getHeadAngle()));
//  									}
							}
							
						    else
							{
							    parser.kick(100, opponentGoalInfo.direction());
							    //System.out.println("OPPONENT GOAL INFO!!!");
							}
						}
					    else     // balldistanz > 0.7
						{
						    if (ballInfo.distance() < 5.0)
							{
							    if(Math.abs(bodyInfo.getHeadAngle()) < 0.5)
								{
								    parser.turnNeck(ballInfo.direction()+ ballInfo.dirChange());
							parser.dash(100.0 * (ballInfo.distance()+ballInfo.distChange()));
								}
							    else
								{
								    parser.turnNeck(-bodyInfo.getHeadAngle());
								    parser.turn((bodyInfo.getHeadAngle())+(ballInfo.direction()));
								}
							}
						    else
							{
							    
							    if (Math.abs(bodyInfo.getHeadAngle()) < 2.0)
								{
//  					    if(ballInfo.distance() + ballInfo.distance() < 2.0)
//  						{ parser.dash(10.0*ballInfo.distance()+ ballInfo.distChange());}

//  					    if(ballInfo.distance()+ ballInfo.distance() < 5.0)
//  						{ parser.dash(30.0*ballInfo.distance()+ballInfo.distChange());}
								    parser.turnNeck(ballInfo.direction() + ballInfo.dirChange());
								    parser.dash(100.0* (ballInfo.distance()+ballInfo.distChange()));
								    
								}
							    else
								{
								    
								    if(bodyInfo.getHeadAngle() < 0)
									{
									    parser.turnNeck(-bodyInfo.getHeadAngle());
									    parser.turn((ballInfo.direction()+ballInfo.dirChange() + bodyInfo.getHeadAngle()));
									}
								    
								    else 
									{
									    parser.turnNeck(-(bodyInfo.getHeadAngle()));
									    parser.turn(ballInfo.direction()+ ballInfo.dirChange()+ bodyInfo.getHeadAngle());
									}
								    
								    
								}
							}
						}    
				}
				
		}

    }




    /**
     * This method contains the strategy of the goalkeeper of our team
     */
    private void playGoalie()
    {
	if(status != free_kick)
	    {	

		if((!ballInfo.name().equals("")) && superPlayer.ballInDefA(ballInfo, actualThreadNumber -1))
		    {
			

			if(ballInfo.distance()+ ballInfo.distChange() <= 1.0)
			    { 

  					if(Math.abs(ballInfo.direction())+ ballInfo.dirChange() < 10.0)
  					    {
  						parser.catchBall(ballInfo.direction()+ ballInfo.dirChange());
  					    }

					else
					    {
						if(ballInfo.distance()+ ballInfo.distChange() <= 0.7) //kickMargin
						    {

					    
							if(selfPosition.getX() != 999.0)
							    {
								parser.turnNeck(ballInfo.direction()+ballInfo.dirChange());
								parser.kick(100, superPlayer.findSpecialKickDirection(playerInfo1,globalView.getTurnDir(selfPosition, 52.5, 0.0,bodyInfo.getHeadAngle()),selfPosition));
								
							    }
							else
							    {
								parser.turnNeck(ballInfo.direction()+ballInfo.dirChange());
								parser.kick(100,opponentGoalInfo.direction());
								//System.out.println("OPPONENT GOAL INFO!!!");
							    }
						    }
						else 
						    {
							parser.turnNeck(-bodyInfo.getHeadAngle());
							parser.turn(ballInfo.direction()+ ballInfo.dirChange());
						    }					    
					    
					    } 
			    }

			    
		    
			else     // balldistanz > 0.7
			    {
				if (ballInfo.distance() < 5.0)
				    {
					if(Math.abs(bodyInfo.getHeadAngle()) < 0.5)
					    {
								    parser.turnNeck(ballInfo.direction()+ ballInfo.dirChange());
								    parser.dash(100.0 * (ballInfo.distance()+ballInfo.distChange()));
					    }
					else
					    {
						parser.turnNeck(-bodyInfo.getHeadAngle());
						parser.turn((bodyInfo.getHeadAngle())+(ballInfo.direction()));
								}
				    }
				else
				    {
					
					if (Math.abs(bodyInfo.getHeadAngle()) < 2.0)
					    {
						//  					    if(ballInfo.distance() + ballInfo.distance() < 2.0)
						//  						{ parser.dash(10.0*ballInfo.distance()+ ballInfo.distChange());}

						//  					    if(ballInfo.distance()+ ballInfo.distance() < 5.0)
						//  						{ parser.dash(30.0*ballInfo.distance()+ballInfo.distChange());}
						parser.turnNeck(ballInfo.direction() + ballInfo.dirChange());
						parser.dash(100.0* (ballInfo.distance()+ballInfo.distChange()));
						
					    }
					else
					    {
						
						if(bodyInfo.getHeadAngle() < 0)
						    {
							parser.turnNeck(-bodyInfo.getHeadAngle());
							parser.turn((ballInfo.direction()+ballInfo.dirChange() + bodyInfo.getHeadAngle()));
						    }
						
						else 
						    {
							parser.turnNeck(-(bodyInfo.getHeadAngle()));
							parser.turn(ballInfo.direction()+ ballInfo.dirChange()+ bodyInfo.getHeadAngle());
						    }
						
						
					    }
				    }
				
			    }//balldistanz >0.7
	    
		    }//if ball seen
		
		else //ball not seen
		    {

			if(superPlayer.checkIfInDefArea(selfPosition, actualThreadNumber) == false)
			    {
				Point2D.Double defAr = superPlayer.getDefAr(actualThreadNumber);
			
				if((zaehler == 0) )
				    {
					
					parser.turn(globalView.getTurnDir(selfPosition,-51.5,0.0, bodyInfo.getHeadAngle()));
					zaehler++;	
				    }
				
				else
				    {
					parser.dash(100*superPlayer.getDistance(selfPosition,-51.5,0.0));
					zaehler = 0;
				    }
				
				
			    }
			else //IN AREA
			    {
				if(ballInfo.name().equals("") || (Math.abs(ballInfo.direction() +ballInfo.dirChange()) > 70.0) )
				    {
					superPlayer.searchBall(ballInfo.direction(),ballInfo.dirChange() ,bodyInfo.getHeadAngle(), ballInfo.distChange());
				    }
				else
				    {
					if(superPlayer.ballInDefA(ballInfo, actualThreadNumber-1)== false)
					    {
						//System.out.println(" BALL NICHT IN MEINER HAELFTE" + actualThreadNumber);      
					    }
					
					
					else //FALL BALL IN MEINEM BEREICH
					    { 
						
						
						if(ballInfo.distance() + ballInfo.distChange() <= 0.7)
						    {
							if(selfPosition.getX() != 999.0)
							    {
								parser.turnNeck(ballInfo.direction()+ ballInfo.dirChange());
								parser.kick(100, superPlayer.findSpecialKickDirection(playerInfo1,globalView.getTurnDir(selfPosition, 52.5, 0.0,bodyInfo.getHeadAngle()),selfPosition));
								
							    }
							else
							    {
								
								parser.turnNeck(ballInfo.direction()+ ballInfo.dirChange());	
								parser.kick(100,opponentGoalInfo.direction());
								//System.out.println("OPPONENT GOAL INFO!!!");
							    }
						    }
						else     // balldistanz > 0.7
						    {


							
								if (ballInfo.distance()+ballInfo.distChange() < 2.0)
								    {
									if(Math.abs(bodyInfo.getHeadAngle()) < 0.5)
									    {
										parser.turnNeck(ballInfo.direction()+ballInfo.dirChange());
										parser.dash(100.0 * (ballInfo.distance()+ballInfo.distChange()));
									    }
									else
									    {
										parser.turnNeck(-bodyInfo.getHeadAngle());
										parser.turn((bodyInfo.getHeadAngle())+(ballInfo.direction()));
									    }
								    }
								else //distance > 10
								    {
									if (Math.abs(bodyInfo.getHeadAngle()) < 2.0)
									    {
										parser.dash(100.0* ballInfo.distance()+ballInfo.distChange());
									    }    
									else
									    {
										
										if(bodyInfo.getHeadAngle() < 0)
										    {
											parser.turnNeck(-bodyInfo.getHeadAngle());
											parser.turn((ballInfo.direction() + bodyInfo.getHeadAngle()));
										    }
										
										else
										    {
											parser.turnNeck(-(bodyInfo.getHeadAngle()));
											parser.turn(ballInfo.direction()+ bodyInfo.getHeadAngle());
										    }
									    }
									
								    }
							    

							
							
						    }
						
						
					    }
				    }
			    }
		    }
		
	    }
	
	else //free_kick status
	    {
		
		if(ballInfo.name().equals("") || (Math.abs(ballInfo.direction() +ballInfo.dirChange()) > 70.0) )
		    {
			superPlayer.searchBall(ballInfo.direction(),ballInfo.dirChange() ,bodyInfo.getHeadAngle(),ballInfo.distChange());
		    }
		else //BALL GEFUNDEN
		    {
			if(ballInfo.distance() + ballInfo.distChange() <= 0.7)
			    {   
				if(selfPosition.getX() != 999.0)
				    {
					parser.kick(100, superPlayer.findSpecialKickDirection(playerInfo1,globalView.getTurnDir(selfPosition, 52.5, 0.0,bodyInfo.getHeadAngle()),selfPosition));
					
				    }
				else
				    {

					parser.kick(100,opponentGoalInfo.direction());
					//System.out.println("OPPONENT GOAL INFO!!!");
				    }
			    }
			else     // balldistanz > 0.7
			    {
				
				if (ballInfo.distance() +ballInfo.distChange() < 10.0)
				    {
					if(Math.abs(bodyInfo.getHeadAngle()) < 0.5)
					    {
						parser.turnNeck(ballInfo.direction()+ballInfo.dirChange());
						parser.dash(100.0 * (ballInfo.distance()+ballInfo.distChange()));
					    }
					else
					    {
						parser.turnNeck(-bodyInfo.getHeadAngle());
						parser.turn((bodyInfo.getHeadAngle())+(ballInfo.direction()));
					    }
				    }
				else //distance > 10
				    {
					if (Math.abs(bodyInfo.getHeadAngle()) < 2.0)
					    {

						parser.turnNeck(ballInfo.direction()+ballInfo.dirChange());
						parser.dash(100.0* (ballInfo.distance()+ballInfo.distChange()));
					    }    
					else
					    {
						
						if(bodyInfo.getHeadAngle() < 0)
						    {
							parser.turnNeck(-bodyInfo.getHeadAngle());
							parser.turn((ballInfo.direction() + bodyInfo.getHeadAngle()));
						    }
						
						else 
						    {
							parser.turnNeck(-(bodyInfo.getHeadAngle()));
							parser.turn(ballInfo.direction()+ bodyInfo.getHeadAngle());
 
						    }
					    }
			    
				    }
			
			    }
		
		    }
	    }
	
    }
    


    /**
     * Starts the client
     *
     * @param args The arguments passed must specify servername port teamname
     * and maybe visualisation when used
     */
    public static void main (String[] args)
    {
	try
	    {
		if (args.length == 3)
		    {
			Player Defense1 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Defense1.setName("D01");
			Defense1.start();

			Player Defense2 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Defense2.setName("D02");
			Defense2.start();

			Player Defense3 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Defense3.setName("D03");
			Defense3.start();

			Player Defense4 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Defense4.setName("D04");
			Defense4.start();

			Player Midfield5 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Midfield5.setName("M05");
			Midfield5.start();

			Player Midfield6 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Midfield6.setName("M06");
			Midfield6.start();

			Player Midfield7 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Midfield7.setName("M07");
			Midfield7.start();

			Player Attack8 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Attack8.setName("A08");
			Attack8.start();

			Player Attack9 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Attack9.setName("A09");
			Attack9.start();

			Player Attack10 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Attack10.setName("A10");
			Attack10.start();

			Player Goalie11 = new Player(args[0], Integer.parseInt(args[1]), args[2]);
			Goalie11.setName("G11");
			Goalie11.start();

		    }
		else
		    {
			if (args.length == 14)
			    {
				Player Defense1 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[3]));
				Defense1.setName("D01");
				Defense1.start();

				Player Defense2 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[4]));
				Defense2.setName("D02");
				Defense2.start();

				Player Defense3 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[5]));
				Defense3.setName("D03");
				Defense3.start();

				Player Defense4 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[6]));
				Defense4.setName("D04");
				Defense4.start();

				Player Midfield5 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[7]));
				Midfield5.setName("M05");
				Midfield5.start();

				Player Midfield6 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[8]));
				Midfield6.setName("M06");
				Midfield6.start();

				Player Midfield7 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[9]));
				Midfield7.setName("M07");
				Midfield7.start();

				Player Attack8 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[10]));
				Attack8.setName("A08");
				Attack8.start();

				Player Attack9 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[11]));
				Attack9.setName("A09");
				Attack9.start();

				Player Attack10 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[12]));
				Attack10.setName("A10");
				Attack10.start();

				Player Goalie11 = new Player(args[0], Integer.parseInt(args[1]), args[2], Integer.parseInt(args[13]));
				Goalie11.setName("G11");
				Goalie11.start();

			    }
			else
			    {
				System.out.println("------------------------------------------------------------------------------");
				System.out.println("Usage :");
				System.out.println("1.      java Player SERVERNAME PORT TEAMNAME");
				System.out.println("  or");
				System.out.println("2.	java Player SERVERNAME PORT TEAMNAME 0 0 0 0 0 0 0 0 0 0 0");
				System.out.println("	(In the second case, the numbers are out of {0,1} and represent");
				System.out.println("	 a boolean value to start (not) the Visualisation for players 1 to 11)");
				System.out.println("");
				System.out.println("SERVERNAME is a STRING");
				System.out.println("PORT is an INTEGER");
				System.out.println("TEAMNAME is a STRING");
				System.out.println("------------------------------------------------------------------------------");
				System.exit(1);
			    }
		    }
	    }
	catch(NumberFormatException e)
	    {
		System.out.println("------------------------------------------------------------------------------");
		System.out.println("Your input has caused a NumberFormatException !");
		System.out.println("");
		System.out.println("Please notice:");
		System.out.println("SERVERNAME is a STRING");
		System.out.println("PORT is an INTEGER");
		System.out.println("TEAMNAME is a STRING");
		System.out.println("Optional parameters are numbers (0 or 1)");
		System.out.println("");
		System.out.println("Usage :");
		System.out.println("1.	java Player SERVERNAME PORT TEAMNAME");
		System.out.println("  or");
		System.out.println("2.	java Player SERVERNAME PORT TEAMNAME 0 0 0 0 0 0 0 0 0 0 0");
		System.out.println("	(In the second case, the numbers are out of {0,1} and represent");
		System.out.println("	 a boolean value to start (not) the Visualisation for players 1 to 11)");
		System.out.println("------------------------------------------------------------------------------");
		System.exit(1);
	    }
    }
}












