/**
 *The class <code>VisualInfo</code> defines a new data type for saving all the needed information 
 *about a seen object. 
 * 
 * @author Andreas Nibler
 */
public class VisualInfo
{
    /** currect cycle */
    private int     time;
    /** name of the seen object */
    private String  name;
    /** distance from the player to the seen object */
    private double  distance;
    /** direction in which the object is seen */
    private double  direction;
    /** approximate change of the distance */
    private double  distChange;
    /** approximate change of the direction */
    private double  dirChange;
    /** x-coordinate of the absolute position of the seen object */
    private double  x=999.0;
    /** y-coordinate of the absolute position of the seen object */
    private double  y=999.0;


    /**
     * Creates an new <code>VisualInfo</code> instance
     *
     *@param time a currect cycle
     *
     *@param name a name of the seen object
     *
     *@param distance a distance from the player to the seen object
     *
     *@param direction a direction in which the object is seen
     *
     *@param distChange an approximate change of the distance
     *
     *@param dirChange an approximate change of the direction
     */
    VisualInfo(int time, String name, double distance, double direction, double distChange, double dirChange)
    {
      this.time=time;
      this.name=name;
      this.distance=distance;
      this.direction=direction;
      this.distChange=distChange;
      this.dirChange=dirChange;
    }


    /**
     *The time when the object has been perceived
     *
     *@return time
     */
    public int time() {return time;}


    /**
     * The name of the object
     *
     *@return name
     */
    public String name() {return name;}


    /**
     * The distance in which the object was perceived
     *
     *@return distance
     */
    public double distance() {return distance;}


    /**
     * The direction in which the object was perceived
     *
     *@return direction
     */
    public double direction() {return direction;}


    /**
     * The approximate change of distance
     *
     *@return distChange
     */
    public double distChange() {return distChange;}


    /**
     * The approximate change of the direction
     *
     *@return dirChange
     */
    public double dirChange() {return dirChange;}


    /**
     * sets the parameter name when the information is not actual
     *(only used for ball)
     */
    public void setName(String name){this.name=name;}


    /**
     * sets the parameter x when calculated
     */
    public void setX(double x){this.x=x;}
   

    /**
     * sets the parameter y when calculated
     */
    public void setY(double y){this.y=y;}

    /**
     * The x-coordinates of the absolute position
     *
     *@return x
     */
    public double getX() {return x;}


    /**
     * The y-coordinates of the absolute position
     *
     *@return y
     */
    public double getY() {return y;}
}





